/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import javax.servlet.http.HttpSession;

import org.mentawai.authorization.Group;
import org.mentawai.core.BaseAction;
import org.mentawai.core.Context;
import org.mentawai.filter.AuthenticationFree;

/**
 * This action implements the basic functionality for a Login action. You should
 * override this class to create your Login action. You should also use its
 * static methods for retrieving the session user and for checking if the user
 * is logged.
 *
 * @author Sergio Oliveira
 */
public abstract class BaseLoginAction extends BaseAction implements
        AuthenticationFree {

    /** The default key used to store the user object in the session. */
    public static final String USER_KEY = "user";

    /**
     * The default key used to store the user authorization groups in the
     * session.
     */
    public static final String GROUPS_KEY = "groups";

    /** The default key used to store the user locale in the session. */
    public static final String LOCALE_KEY = "locale";

    /**
     * Place an user object in the session. Note that this can be any object.
     * <br>
     * <br>
     * <b><font color="red">Attention:</font>The session context is reset when
     * you call this method.</b>
     *
     * @param value
     *            The user object to place in the session.
     */
    public void setUserSession(Object value) {
        setUserSession(value, session);
    }

    /**
     * Place an user object in the session. Note that this can be any object.
     * <br>
     * <br>
     * <b><font color="red">Attention:</font>The session context is reset when
     * you call this method.</b>
     * ]
     * @param value
     *            The user object to place in the session.
     * @param session
     *            The session where to put the user.
     * @since 1.1.1
     */
    public static void setUserSession(Object value, Context session) {
        session.reset();
        session.setAttribute(USER_KEY, value);
    }

    /**
     * Place the user locale in the session. If you call this method, the web
     * application will follow this locale and not the one sent by the browser.
     *
     * @param loc
     *            The user locale to place in the session.
     */
    public void setUserLocale(Locale loc) {
        setUserLocale(loc, session);
    }

    /**
     * Place the user locale in the session. If you call this method, the web
     * application will follow this locale and not the one sent by the browser.
     *
     * @param loc
     *            The user locale to place in the session.
     * @param session
     *            The session where to put the user locale.
     * @since 1.1.1
     */
    public static void setUserLocale(Locale loc, Context session) {

        if (!session.hasAttribute(USER_KEY)) {

            throw new IllegalStateException(
                    "Tried to set user locale, but there is no user in the session!");
        }

        session.setAttribute(LOCALE_KEY, loc);
    }

    /**
     * Place the user authorization groups in the session.
     *
     * @param groups
     *            The user authorization groups to place in the session.
     */
    public void setUserGroups(List groups) {
        setUserGroups(groups, session);
    }

    /**
     * Place the user authorization groups in the session.
     *
     * @param groups
     *            The user authorization groups to place in the session.
     * @param session
     *            The session where to put the authorization groups.
     * @since 1.1.1
     */
    public static void setUserGroups(List groups, Context session) {

        if (!session.hasAttribute(USER_KEY)) {

            throw new IllegalStateException(
                    "Tried to set user groups, but there is no user in the session!");
        }

        List<String> list = new ArrayList<String>(groups.size());
        Iterator iter = groups.iterator();
        while (iter.hasNext()) {
            Object obj = iter.next();
            if (obj instanceof String) {
                String s = (String) obj;
                list.add(s);
            } else if (obj instanceof Group) {
                Group g = (Group) obj;
                list.add(g.getName());
            }
        }
        session.setAttribute(GROUPS_KEY, list);
    }

    /**
     * Place the user authorization groups in the session.
     *
     * @param groups
     *            The user authorization groups to place in the session.
     */
    public void setUserGroups(String groups) {
        setUserGroups(groups, session);
    }

    /**
     * Place the user authorization groups in the session.
     *
     * @param groups
     *            The user authorization groups to place in the session.
     * @param session
     *            The session where to put the authorization groups.
     * @since 1.1.1
     */
    public static void setUserGroups(String groups, Context session) {

        if (!session.hasAttribute(USER_KEY)) {

            throw new IllegalStateException(
                    "Tried to set user groups, but there is no user in the session!");
        }

        StringTokenizer st = new StringTokenizer(groups, ",");
        List<String> list = new ArrayList<String>(st.countTokens());
        while (st.hasMoreTokens()) {
            list.add(st.nextToken().trim());
        }
        session.setAttribute(GROUPS_KEY, list);
    }

    /**
     * Check if the user is logged.
     *
     * @return true if the user is logged.
     */
    public boolean isLogged() {
        return isLogged(session);
    }

    /**
     * Return the user session object.
     *
     * @return The user session or null if the user is not logged.
     */
    public Object getUserSession() {
        return getUserSession(session);
    }

    /**
     * Return the user locale.
     *
     * @return The user locale or null if the user is not logged.
     */
    public Locale getUserLocale() {
        return getUserLocale(session);
    }

    /**
     * Return the user authorization groups.
     *
     * @return The user authorization groups or null if the user is not logged
     *         or user has no authorization groups.
     */
    public List getUserGroups() {
        return getUserGroups(session);
    }

    /**
     * Check if the user is logged.
     *
     * @param session
     *            The session context where to check for the user object.
     * @return true if the user is logged.
     */
    public static boolean isLogged(Context session) {
        return session.hasAttribute(USER_KEY);
    }

    /**
     * Check if the user is logged.
     *
     * @param session
     *            The http session where to check for the user object.
     * @return true if the user is logged.
     */
    public static boolean isLogged(HttpSession session) {
        return session.getAttribute(USER_KEY) != null;
    }

    /**
     * Return the user session object.
     *
     * @param session
     *            The session context where to check for the user object.
     * @return The user session or null if the user is not logged.
     */
    public static Object getUserSession(Context session) {
        return session.getAttribute(USER_KEY);
    }

    /**
     * Return the user session object.
     *
     * @param session
     *            The session context where to check for the user object.
     * @return The user session or null if the user is not logged.
     */
    public static Object getUserSession(HttpSession session) {
        return session.getAttribute(USER_KEY);
    }

    /**
     * Return the user locale.
     *
     * @param session
     *            The session context where to check for the user locale.
     * @return The user locale or null if the user is not logged.
     */
    public static Locale getUserLocale(Context session) {
        return (Locale) session.getAttribute(LOCALE_KEY);
    }

    /**
     * Return the user locale.
     *
     * @param session
     *            The session context where to check for the user locale.
     * @return The user locale or null if the user is not logged.
     */
    public static Locale getUserLocale(HttpSession session) {
        return (Locale) session.getAttribute(LOCALE_KEY);
    }

    /**
     * Return the user authorization groups.
     *
     * @param session
     *            The session context where to check for the user authorization
     *            groups.
     * @return The user authorization groups or null if the user is not logged
     *         or the user has no authorization groups.
     */
    public static List getUserGroups(Context session) {
        return (List) session.getAttribute(GROUPS_KEY);
    }

    /**
     * Return the user authorization groups.
     *
     * @param session
     *            The http session where to check for the user authorization
     *            groups.
     * @return The user authorization groups or null if the user is not logged
     *         or the user has no authorization groups.
     */
    public static List getUserGroups(HttpSession session) {
        return (List) session.getAttribute(GROUPS_KEY);
    }
}
