/**
 * 
 */
package org.mentawai.ajax.renders;

import org.mentawai.ajax.AjaxRender;
import org.mentawai.core.Action;
import org.mentawai.core.Output;

import com.thoughtworks.xstream.XStream;

/**
 * A AjaxRender that renderizes the Action using the XStream framework.
 * 
 * @author Rubem Azenha (rubem.azenha@gmail.com)
 */
public class XStreamRender implements AjaxRender {

    /**
     * The name of the attribute of the action's output that will be renderized
     * using XStream.
     */
    private String attribute;

    /**
     * The XStream objet that will serialize a action's output attribute into
     * XML.
     */
    private XStream xstream;

    /**
     * Creates a XStreamRender, defining the name of the attribute that will be
     * serialized into XML by the XStream and using the default XStream object
     * (new XStream).
     * 
     * @param attribute
     *            the name of the attribute that will be serialized into XML by
     *            the XStream.
     */
    public XStreamRender(String attribute) {

        this(attribute, new XStream());
    }

    /**
     * Creates a XStreamRender, defining the name of the attribute that will be
     * serialized into XML by the XStream and the XStream object.
     * 
     * @param attribute
     *            the name of the attribute that will be serialized into XML by
     *            the XStream.
     * @param xstream
     *            the XStream object.
     */
    public XStreamRender(String attribute, XStream xstream) {

        if (attribute == null) {
            throw new IllegalArgumentException(
                    "The attribute \'attribute\' cannot be null!");
        }
        this.attribute = attribute;

        if (xstream == null) {
            throw new IllegalArgumentException(
                    "The attribute \'xStream\' cannot be null!");
        }
        this.xstream = xstream;
    }

    /**
     * Renderizes the Action using XStream.
     * 
     * @return the result of the action renderization.
     * @throws Exception
     *             if something goes wrong.
     */
    public String renderize(Action action) throws Exception {
        Output output = action.getOutput();
        Object attribute = output.getValue(this.attribute);
        return xstream.toXML(attribute);
    }

    public String getContentType() {
        return TEXT_XML;
    }

}
