/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.core;

import java.util.Locale;

/**
 * Describes a Mentawai action, the central idea of the framework architecture.
 * An action has an input (org.mentawai.core.Input) and an output (org.mentawai.core.Output).
 * An action generates a result (java.lang.String) after it is executed. The result is usually SUCCESS or ERROR.
 * For each result there is a consequence (org.mentawai.core.Consequence). The consequences for a web application are usually FORWARD or REDIRECT.
 * An action has access to contexts (org.mentawai.core.Context). The contexts for a web application are usually a SessionContext or a ApplicationContext.
 *
 * @author Sergio Oliveira
 */
public interface Action {
    
    /** The action execution was successful. */    
   	public static final String SUCCESS = "success";
    
    /** The action execution generated an error. */
	public static final String ERROR = "error";
	
	/**
	 * Executes the action, returning a result.
	 * 
	 * @return a string with the result
	 * @throws Exception if these action cannot be executed for some reason
	 */
	public String execute() throws Exception;
	
	/**
	 * Sets the input for this action.
	 * 
	 * @param input the input to be set
	 */
	public void setInput(Input input);
	
	/**
	 * Sets the output for this action.
	 * 
	 * @param output the output to be set
	 */
	public void setOutput(Output output);
	
	/**
	 * Sets the session context for this action.
	 * 
	 * @param context The context to be set
	 */
	public void setSession(Context context);
	
	/**
	 * Sets the application context for this action.
	 * 
	 * @param context The context to be set
	 */
	public void setApplication(Context context);
    
    /**
     * Sets the cookie context for this action.
     * 
     * @param context The cookie context to be set
     */
    public void setCookies(Context context);
	
	/**
	 * Sets the user locale for this action.
	 *
	 * @param loc The user locale to set.
	 */
	 public void setLocale(Locale loc);
     
	/**
	 * Gets the action input.
	 * 
	 * @return The action input
	 */
	public Input getInput();
	
	/**
	 * Gets the action output.
	 * 
	 * @return The action output
	 */
	public Output getOutput();
	
	/**
	 * Gets this action session context.
	 * 
	 * @return The session context
	 */
	public Context getSession();
	
	/**
	 * Gets this action application context.
	 * 
	 * @return The application context
	 */
	public Context getApplication();
    
    /**
     * Gets the cookie context for this action.
     * 
     * @return The cookie context
     * @since 1.2
     */
    public Context getCookies();
	
	/**
	 * Gets the use locale for this action.
	 *
	 * @return The user locale
     * @since 1.2
	 */
	 public Locale getLocale();
}

	