/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.core;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;

import org.mentawai.util.DebugServletFilter;

/**
 * When an action is executed, a chain of filters is created.
 * The last step of any InvocationChain is the action.
 * An action may have one or more filters and global filters.
 * 
 * @author Sergio Oliveira
 */
public class InvocationChain {

	private LinkedList<Filter> filters = new LinkedList<Filter>();
	private Action action;
    private String innerAction = null;
    private final String actionName;

	/**
	 * Creates an InvocationChain for this action.
	 * 
	 * @param action The action for what this InvocationChain will be created.
	 */
	public InvocationChain(String actionName, Action action) {
		
		this.actionName = actionName;
		
		this.action = action;
	}

	/**
	 * Gets the action of this InvocationChain
	 * 
	 * @return The action of this InvocationChain
	 */
	public Action getAction() {
		return action;
	}

	void addFilter(Filter filter) {
		filters.add(filter);
	}

	void addFilters(List<Filter> list) {
		filters.addAll(list);
	}
    
	/**
	 * Invoke and execute the next step in this InvocationChain.
	 * This can be the next filter or the action.
	 * 
	 * @return The result of a filter or the action.
	 */
	public String invoke() throws Exception {
        
        if (Controller.debugMode) DebugServletFilter.debugInputOutput(action);
            
		if (!filters.isEmpty()) {
			Filter f = (Filter) filters.removeFirst();
            
            if (Controller.debugMode) {
                
                StringBuffer sb = DebugServletFilter.getDebug(action);
                
                DebugServletFilter.debug(sb, f);
                
            }
            
			return f.filter(this);
		}
        
        if (Controller.debugMode) {
            
            StringBuffer sb = DebugServletFilter.getDebug(action);
            
            DebugServletFilter.debug(sb, innerAction, false);
            
        }
        
        String result = null;
        
		if (innerAction != null) {
			
			// first try an inner class...
			
			Class<Action> klass = getInnerClass(innerAction);
			
			if (klass != null) {
				
				// get constructor...
				
				Constructor c = getConstructor(klass);
				
				if (c == null) throw new ActionException("The inner class for this inner action does not have a constructor: " + innerAction);
				
				// ok, now check whether this is a static inner class or instance inner class...
				
				Class[] paramTypes = c.getParameterTypes();
				
				if (paramTypes.length == 0) {
					
					// static inner class...
					
					try {
						
						Action innerActionClass = klass.newInstance();
						
						initInnerAction(action, innerActionClass);
						
						result = innerActionClass.execute();
						
					} catch(Exception e) {
						
						throw new ActionException(e);
					}
					
				} else if (paramTypes.length == 1) {
					
					// instance inner class...
					
					try {
						
						Action innerActionClass =  (Action) c.newInstance(action);
						
						initInnerAction(action, innerActionClass);
						
						result = innerActionClass.execute();
						
					} catch(Exception e) {
						
						throw new ActionException(e);
					}
					
				}
				
			} else {
			
				Method m = getMethod(innerAction);
				
				if (m != null) {
					try {
						result = (String) m.invoke(action, new Object[0]);
					} catch(Exception e) {
						throw new ActionException(e);
					}
				} else {
					throw new ActionException("The inner action does not exist: " + innerAction);
				}
			
			}
		} else {
			result = action.execute();
		}
        
        return result;
	}
	
	/**
	 * Initialize inner action class contexts with the main action contexts...
	 *  
	 * @param mainAction The main action.
	 * @param innerAction The inner action object. (inner class)
	 * @since 1.9
	 */
	protected void initInnerAction(Action mainAction, Action innerAction) {
		
		innerAction.setInput(mainAction.getInput());
		innerAction.setOutput(mainAction.getOutput());
		innerAction.setSession(mainAction.getSession());
		innerAction.setApplication(mainAction.getApplication());
		innerAction.setCookies(mainAction.getCookies());
		innerAction.setLocale(mainAction.getLocale());
		
	}
	
	private Constructor getConstructor(Class<Action> klass)  {
		
		Constructor c = null;
		
		try {
		
			c = klass.getConstructor((Class[]) null);
			
		} catch(NoSuchMethodException e) {

			// do nothing... not found...
			
		}
			
		if (c != null) return c;
		
		try {
			
			c = klass.getConstructor(action.getClass());
			
		} catch(NoSuchMethodException e) {
			
			// do nothing... not found...
			
		}
		
		return c;
		
	}
	
	private Class<Action> getInnerClass(String innerAction) {
		
		Class[] classes = action.getClass().getClasses();
		
		for (Class clazz : classes) {
			
			String simpleName = clazz.getSimpleName();
			
			if(Action.class.isAssignableFrom(clazz) && (simpleName.equals(innerAction) || simpleName.equalsIgnoreCase(innerAction))){
				
				return (Class<Action>) clazz;
				
			}
		}
		
		return null;
	}
	
	private Method getMethod(String innerAction) {
		try {
			Method m = action.getClass().getMethod(innerAction, new Class[0]);
			if (m != null) {
				return m;
			}			
		} catch(Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Sets an inner action to be executed.
	 * An inner action is a method inside the action implementation that can be executed instead of the execute() method.
	 *
	 * @param innerAction The name of the method to be executed as an inner action
	 */
	public void setInnerAction(String innerAction) {
		this.innerAction = innerAction;
	}
    
    /**
     * Returns the inner action being executed in the invocation chain.
     *
     * @return The innner action or null if there is no inner action being executed.
     * @since 1.2.1
     */
    public String getInnerAction() {
        return innerAction;
    }

    /**
     * Returns the name of the action being executed in the invocation chain.
     * 
     * @return The action name like HelloMentawai
     * @since 1.8
     */
    public String getActionName() {
    	return actionName;
    }
    
    /**
     * Returns the filters of this invocation chain.
     * 
     * @return all filters of this invocation chain
     * @since 1.4
     */
    public List<Filter> getFilters() {
        
        return filters;
        
    }
        
}