package org.mentawai.core;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Marvin H. Froeder
 */
public abstract class MultiApplicationManager extends ApplicationManager {

	private final List<ApplicationManager> managers;
	
	private boolean initialized = false;

	/**
	 * Default constructor, capable to call the registerManagers() method.
	 * 
	 * @throws Exception
	 *             about the ApplicationManager instanciation
	 */
	public MultiApplicationManager() {
		super();
		this.managers = new ArrayList<ApplicationManager>();
		registerManagers();
		initialized = true;
	}

	/**
	 * Abstract method to alow the registration the ApplicationManagers.
	 * 
	 * @throws Exception
	 *             about the ApplicationManager instanciation
	 */
	public abstract void registerManagers();

	/**
	 * Call this method to register an ApplicationManager.
	 * 
	 * @param manager The application manager to register.
	 */
	public void register(Class<? extends ApplicationManager> manager) {
		
		if (initialized) throw new IllegalStateException("MultiApplicationManager is already initialized! Call register from registerManagers() method!");
		
		try {
			ApplicationManager newInstance = manager.newInstance();
			newInstance.setParent(this);
			this.managers.add(newInstance);
		} catch (Exception e) {
			throw new RuntimeException("Unable to instanciate the class: "
					+ manager.getSimpleName() + ".  Read the next stack for details.", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mentawai.core.ApplicationManager#init(org.mentawai.core.Context)
	 */
	@Override
	public final void init(Context application) {
		super.init(application);
		for (ApplicationManager manager : this.managers) {
			manager.init(application);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mentawai.core.ApplicationManager#loadActions()
	 */
	@Override
	public final void loadActions() {
		super.loadActions();
		for (ApplicationManager manager : this.managers) {
			manager.loadActions();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mentawai.core.ApplicationManager#loadLocales()
	 */
	@Override
	public final void loadLocales() {
		super.loadLocales();
		for (ApplicationManager manager : this.managers) {
			manager.loadLocales();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mentawai.core.ApplicationManager#loadFormatters()
	 */
	@Override
	public final void loadFormatters() {
		super.loadFormatters();
		for (ApplicationManager manager : this.managers) {
			manager.loadFormatters();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mentawai.core.ApplicationManager#loadLists()
	 */
	@Override
	public final void loadLists() throws IOException {
		super.loadLists();
		for (ApplicationManager manager : this.managers) {
			manager.loadLists();
		}
	}
}
