/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.core;

import java.net.URI;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A redirect web consequence.
 * A normal redirect web consequence has just a url.
 * A redirect may also have dynamic parameters and a dynamic url.
 * The following rules govern the interpretation of URLs:
 * <ol>
 * <li>Paths starting with '//' are relative to the webserver root.
 * <li>Paths starting with '/' are relative to the context path.
 * <li>Other paths are relative to the current request.
 * </ol>
 * @author Alan Burlison Alan.Burlison@sun.com
 */
public class Redirect implements Consequence {

    /** The parameter's name in the output for the dynamic redirect. */
    public static final String REDIRURL_PARAM = "_mentawai_redirect_url";

    /** The target URL of the redirect. */
    private String url = null;

    /** True if the {@link Output} values are to be appended to the URL. */
    private boolean appendOutput = false;

    /** True if the URL is to be obtained from the {@link Output}. */
    private boolean dynamicUrl = false;

    /**
     * Creates a normal redirect for the given url.  The supplied URL
     * fully specifies the target of the redirect.
     * @param url is the url for this redirect.
     */
    public Redirect(String url) {
        this.url = url;
    }

    /**
     * Creates a redirect for the given url with dynamic parameters.
     * The URL query parameters are obtained from the {@link Output} object
     * for the {@link Action}.  All the parameters are appended to the URL query string.
     * 
     * @param url is the url for this redirect.
     * @param appendOutput is true if this redirect is to use dynamic
     * parameters.
     */
    public Redirect(String url, boolean appendOutput) {
        this.url = url;
        this.appendOutput = appendOutput;
    }

    /**
     * Creates a redirect with a dynamic url.
     * 
     * The redirect url is obtained from the action output
     * through the key REDIRURL_PARAM.
     *
     */
    public Redirect() {
        this.dynamicUrl = true;
    }

    /**
     * Creates a redirect with a dynamic URL and dynamic parameters.
     * 
     * The redirect url is obtained from the action output.
     * 
     * The URL query parameters are obtained from the {@link Output} object
     * for the {@link Action}.  All the parameters are appended to the URL query string.
     * @param appendOutput is true if this redirect is to use dynamic
     * parameters.
     */
    public Redirect(boolean appendOutput) {
        this.dynamicUrl = true;
        this.appendOutput = appendOutput;
    }

    /**
     * Execute the redirect consequence.  The URL to redirect to is built
     * up as necessary, depending on the type of redirection that is
     * required - see the constructirs for this class for the various
     * flavours of redirections that are supported.
     * @param act action to be redirected.
     * @param req request object.
     * @param res response object
     * @throws ConsequenceException if an error is detected during
     * construction of the redirection URL.
     */
    public void execute(Action act, HttpServletRequest req, HttpServletResponse res) throws ConsequenceException {
        try {
            
            Output output = act != null ? act.getOutput() : null;

            // Determine the URL to redirect to.
            String path;
            if (dynamicUrl && output != null) {
                path = (String) output.getValue(REDIRURL_PARAM);
            } else {
                path = url;
            }
            
            if (path == null || path.length() == 0) {
                    throw new ConsequenceException(
                      "Missing url for redirect");
            }

            // Double-slash paths are relative to the server root.
            URI reqUri = new URI(req.getRequestURL().toString());
            if (path.length() >= 2 && path.startsWith("//")) {
                URI u = reqUri.resolve(path.substring(1));
                path = u.getPath();

            // Single-slash paths are relative to the context path.
            } else if (path.length() >= 1 && path.startsWith("/")) {
                URI u = reqUri.resolve(req.getContextPath());
                StringBuffer s = new StringBuffer(u.getPath());
                if (s.charAt(s.length() - 1) == '/') {
                    s.append(path.substring(1));
                } else {
                    s.append(path);
                }
                path = s.toString();
            
            // Otherwise it's a normal URI.
            } else {
                    URI u = reqUri.resolve(path);
                    path = u.toASCIIString();
            }

            // Get any existsing query string.
            URI uri = new URI(path);
            StringBuffer query = new StringBuffer();
            String q = uri.getQuery();
            if (q != null) {
                query.append(q);
            }

            // If we have dynamic parameters, append them to the query string.
            if (appendOutput && act != null) {

                // Build the parameters into a query string.
                Iterator<String> iter = act.getOutput().keys();
                while (iter.hasNext()) {
                    String key = (String) iter.next();

                    // Skip the REDIRURL_PARAM, if present.
                    if (key.equals(REDIRURL_PARAM)) {
                        continue;
                    }

                    // Skip null values.
                    Object value = output.getValue(key);
                    if (value == null) {
                        continue;
                    }

                    // Append the key and value.
                    if (query.length() > 0) {
                        query.append('&');
                    }
                    query.append(key);
                    query.append("=");
                    query.append(value);
                }
            }

	    // Create the final URL and do the redirect.
            uri = new URI(uri.getScheme(), uri.getUserInfo(),
                uri.getHost(), uri.getPort(), uri.getPath(),
                query.length() == 0 ? null : query.toString(),
                uri.getFragment());
            res.sendRedirect(uri.toASCIIString());
        } catch(Exception e) {
            throw new ConsequenceException(e);
        }
    }

    /**
     * Return a string representation of the redirect.
     * @return a string representation of the redirect.
     */
    public String toString() {
        StringBuffer s = new StringBuffer();
        s.append("Redirect to ");
        if (url != null) s.append(url);
        else s.append("(dynamic redirect)");
        return s.toString();
    }
}