/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.mentawai.action.BaseLoginAction;
import org.mentawai.ajax.AjaxAction;
import org.mentawai.authorization.AuthorizationManager;
import org.mentawai.authorization.Group;
import org.mentawai.authorization.Permission;
import org.mentawai.core.Action;
import org.mentawai.core.Context;
import org.mentawai.core.Filter;
import org.mentawai.core.InvocationChain;

/**
 * A filter to handle user authorization.
 * You should use this filter to protect your actions from unauthorized access.
 * 
 * @author Sergio Oliveira
 */
public class AuthorizationFilter implements Filter {
	
	public static final String ACCESSDENIED = "accessdenied";
	public static final String AJAX_DENIED = AuthenticationFilter.AJAX_DENIED;
    
    private List<String> groups = null;
    private String permission = null;
	
    /**
     * Creates a new authorization filter.
     */
	public AuthorizationFilter(String groups) {
        StringTokenizer st = new StringTokenizer(groups, ",");
        this.groups = new ArrayList<String>(st.countTokens());
        while(st.hasMoreTokens()) {
            this.groups.add(st.nextToken().trim());
        }
    }
    
    public AuthorizationFilter(List groups) {
        this.groups = new ArrayList<String>(groups.size());
        Iterator iter = groups.iterator();
        while(iter.hasNext()) {
            Object obj = iter.next();
            if (obj instanceof Group) {
                Group g = (Group) obj;
                this.groups.add(g.getName());
            } else if (obj instanceof String) {
                String s = (String) obj;
                this.groups.add(s);
            }
        }
    }
    
    public AuthorizationFilter(Permission permission) {
        this.permission = permission.getName();
    }
    
    public AuthorizationFilter(String groups, Permission permission) {
        this(groups);
        this.permission = permission.getName();
    }
    
    public AuthorizationFilter(List groups, Permission permission) {
        this(groups);
        this.permission = permission.getName();
    }
	
	public String filter(InvocationChain chain) throws Exception {
        Action action = chain.getAction();
        Context session = action.getSession();
        
        List usergroups = BaseLoginAction.getUserGroups(session);
        if (usergroups == null || usergroups.size() == 0) {
        	
        	if (action instanceof AjaxAction) {
        		
        		return AJAX_DENIED;
        	}
        	
			return ACCESSDENIED;
		}
        
        boolean ok = false;
        
        if (groups != null && groups.size() > 0) {
            for(int i=0;i<groups.size();i++) {
                for(int j=0;j<usergroups.size();j++) {
                    String group = groups.get(i).toString();
                    String usergroup = usergroups.get(j).toString();
                    if (usergroup.equals(group)) {
                        ok = true;
                        i = groups.size();
                        j = usergroups.size();
                    }
                }
            }
            if (!ok) {
            	
            	if (action instanceof AjaxAction) {
            		
            		return AJAX_DENIED;
            	}
            	
                return ACCESSDENIED;
            }
        }
        
        if (permission != null && permission.length() > 0) {
            
            Iterator iter = usergroups.iterator();
            while(iter.hasNext()) {
                String usergroup = iter.next().toString();
                if (AuthorizationManager.check(usergroup, permission)) {
                    ok = true;
                    break;
                }
            }
            if (!ok) {
            	
            	if (action instanceof AjaxAction) {
            		
            		return AJAX_DENIED;
            		
            	}
            	
                return ACCESSDENIED;
            }
        }
        
        if (!ok) {
        	
        	if (action instanceof AjaxAction) {
        		
        		return AJAX_DENIED;
        		
        	}
        	
            return ACCESSDENIED;
        }

		return chain.invoke();
	}
	
    public void destroy() { }
	
}
		