/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.mentawai.core.Action;
import org.mentawai.core.Filter;
import org.mentawai.core.FilterException;
import org.mentawai.core.InvocationChain;
import org.mentawai.util.InjectionUtils;

/**
 * This filter will inject input values into the action or model.
 *
 * @author Sergio Oliveira
 */
public class InjectionFilter implements Filter {
    
    private Map setters = new HashMap();
    private Map fields = new HashMap();
    private boolean tryField = true;
    private boolean convert = true;
    
    /**
     * Creates a InjectionFilter that can be used by any action class.
     * You may use this filter per action or as a global filter.
     */
    public InjectionFilter() { }
    
    /**
     * Creates an InjectionFilter that can be used by any action class.
     * You may use this filter per action or as a global filter.
     * If tryField is true and it cannot find a setter for the input value,
     * it will try to directly access the attribute, even if it is a private field.
     *
     * @param tryField A flag indicating whether this filter should try to access private attributes.
     */    
    public InjectionFilter(boolean tryField) {
        this.tryField = tryField;
    }
    
    /**
     * Creates an InjectionFilter that can be used by any action class.
     * You may use this filter per action or as a global filter.
     * If tryField is true and it cannot find a setter for the input value,
     * it will try to directly access the attribute, even if it is a private field.
     * If convert flag is true (default), it will try to automatically convert.
     *
     * @param tryField A flag indicating whether this filter should try to access private attributes.
     */    
    public InjectionFilter(boolean tryField, boolean convert) {
        this(tryField);
        this.convert = convert;
    }    
    
    private boolean inject(Method m, Object target, Object value, Locale loc) throws FilterException {
        
        Class type = m.getParameterTypes()[0];
        
        if (type.isAssignableFrom(value.getClass()) || InjectionUtils.checkPrimitives(type, value.getClass()) || (convert && (value = InjectionUtils.tryToConvert(value, type, loc)) != null)) {
            
            try {
            
                m.invoke(target, new Object[] { value });
                
                return true;
                
            } catch(Exception e) {
                
                System.err.println("Error injecting " + value + " in " + target);
                
                e.printStackTrace();
                
                throw new FilterException(e);
                
            }
        }
        
        return false;
        
    }
    
    protected Object getTarget(Action action) throws FilterException {
        
        if (action instanceof ModelDriven) {
            
            ModelDriven md = (ModelDriven) action;
            
            Object model = md.getModel();
            
            if (model == null) throw new FilterException("ModelDriven action cannot return a null model!");
            
            return model;
            
        }
        
        return action;
    }
    
    public String filter(InvocationChain chain) throws Exception {
    	
    	Action action = chain.getAction();
    	
    	Object target = getTarget(action);
    	
    	InjectionUtils.getObject(target, action.getInput(), action.getLocale(), tryField, null, convert);
    	
    	return chain.invoke();
    }
    
    public void destroy() { }
}
        