/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.lang.reflect.Method;

import org.mentawai.core.Action;
import org.mentawai.core.Filter;
import org.mentawai.core.InvocationChain;
import org.mentawai.core.Output;

/**
 * A filter that takes all the properties of the action and place them in the output,
 * so that you don't have to call output.setValue() manually.
 * 
 *<br>
 * Note that this filter uses the method <i>action.getClass().getDeclaredMethods()</i> in order to find the getters, in other words,
 * it will only find getters of the base class and not from its superclasses. This is ok because
 * action inheritance is not very common.
 * <BR>
 * For ModelDriven actions, the method <i>action.getModel().getClass().getMethods()</i> is used instead, pretty much like in the OVFilter.
 *
 * @author Sergio Oliveira
 */
public class OutputFilter implements Filter {
	
    /**
     * Creates a OutputFilter.
     */
	public OutputFilter() { }
        
    
    private String adjustName(String name) {
        StringBuffer sb = new StringBuffer(name.length() - 3);
        sb.append(name.substring(3, 4).toLowerCase());
        sb.append(name.substring(4, name.length()));
        return sb.toString();
    }
    
	public String filter(InvocationChain chain) throws Exception {
        
        String result = chain.invoke();
        
		Action action = chain.getAction();
        Output output = action.getOutput();
        
        boolean isModelDriven = false;
        if (action instanceof ModelDriven) {
            isModelDriven = true;
        }
        
        Method [] methods = null;
        if (isModelDriven) {
            ModelDriven md = (ModelDriven) action;
            methods = md.getModel().getClass().getMethods();
        } else {
            methods = action.getClass().getDeclaredMethods();
        }

        for(int i=0;i<methods.length;i++) {
            String name = methods[i].getName();
            if (name.length() > 3 && name.startsWith("get") && methods[i].getParameterTypes().length == 0) {
                if (!isModelDriven && name.equals("getClass")) continue;
                try {
                    methods[i].setAccessible(true);
                    
                    Object value = null;
                    
                    if (isModelDriven) {
                        
                        ModelDriven md = (ModelDriven) action;
                        
                        value = methods[i].invoke(md.getModel(),  new Object[0]);
                        
                    } else {
                    
                        value = methods[i].invoke(action,  new Object[0]);
                        
                    }

                    output.setValue(adjustName(name), value);
                    
                } catch(Exception e) {
                    System.err.println("Error calling method in OutputFilter: " + name);
                    e.printStackTrace();
                }
            }
        }
		return result;
	}
    
    public void destroy() { }
}
		