/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.mentawai.core.Action;
import org.mentawai.core.BaseAction;
import org.mentawai.core.Filter;
import org.mentawai.core.InvocationChain;
import org.mentawai.i18n.LocaleManager;
import org.mentawai.message.ClassMessageContext;
import org.mentawai.message.FileMessageContext;
import org.mentawai.message.MessageContext;
import org.mentawai.rule.Rule;
import org.mentawai.validation.Validator;

/**
 * A filter to validate the values of an action input.
 * Apply this filter to validate the input values of your actions.
 * By default, error messages are inside the <i>/validation/</i> directory and a ClassMessageContext is used. (Only if LocaleManager.isUseMasterForEverything returns false)
 * By default, the result returned when the validation fails is <i>BaseAction.ERROR</i>.
 *
 * @author Sergio Oliveira
 */
public abstract class ValidationFilter implements Filter {
	
	private static final String DEFAULT_DIR = "/validation";
	
	private Validator validator;
	private Map<String, Object> rules = new HashMap<String, Object>();
	private Map<String, Object> errors = new HashMap<String, Object>();
	
	private boolean isLoaded = false;
	
	private String resultForError = BaseAction.ERROR;
	private MessageContext msgContext;
	
	private boolean isDynamic = false;
	
	private ThreadLocal<Map<String, Object>> tRules = null;
	private ThreadLocal<Map<String, Object>> tErrors = null;
	private ThreadLocal<Action> tAction = new ThreadLocal<Action>();
	
    /**
     * Creates a ValidationFilter.
     * This constructor calls the <i>initValidator()</i> method.
     */
	public ValidationFilter() {
        
        if (LocaleManager.isUseMasterForEverything()) {
            msgContext = new FileMessageContext(LocaleManager.getMaster(), "");
        } else {
            msgContext = new ClassMessageContext(this.getClass(), DEFAULT_DIR);
        }
	}
	
	public ValidationFilter(boolean isDynamic) {
        
		this();
		
		this.isDynamic = isDynamic;
		
		if (isDynamic) {
			
			 tRules = new ThreadLocal<Map<String, Object>>();
			 tErrors = new ThreadLocal<Map<String, Object>>();
		}
		
	}

	
	private Validator getValidator() {
		
		if (!isDynamic) {
		
			if (isLoaded) return validator;
			
			synchronized(this) {
				
				if (!isLoaded) {
					
					initValidator();
					
					validator = new Validator(rules, errors);
					
					isLoaded = true;
				}
			}
			
			return validator;
		
		} else {
			
			Map<String, Object> rules = new HashMap<String, Object>();
			Map<String, Object> errors = new HashMap<String, Object>();
			
			tRules.set(rules);
			tErrors.set(errors);
			
			initValidator();
			
			return new Validator(rules, errors);
		}
	}
	
    /**
     * Adds a validation rule to the given action input field.
     * If the validation fails, show the error message specified by the given error id.
     *
     * @param field The action input field to validate.
     * @param rule The validation rule to apply.
     * @param error_id The error message to show in case of failure.
     */
	public void add(String field, Rule rule, int error_id) {
        add(field, rule, String.valueOf(error_id));
	}
    
    /**
     * Adds a validation rule to the given action input field.
     * If the validation fails, show the error message specified by the given error id.
     *
     * @param field The action input field to validate.
     * @param rule The validation rule to apply.
     * @param error_id The error message to show in case of failure.
     * @since 1.1.1
     */
	public void add(String field, Rule rule, String error_id) {
		
		Map<String, Object> rules, errors;
		
		if (isDynamic) {
			
			// if dynamic fetch from ThreadLocal...
			
			rules = tRules.get();
			errors = tErrors.get();
			
		} else {
			
			rules = this.rules;
			errors = this.errors;
		}
		
		// add the rule...
		List<Rule> list = (List<Rule>) rules.get(field);
		if (list == null) {
			list = new LinkedList<Rule>();
			rules.put(field ,list);
		}
		list.add(rule);
		
		// add the error id...
		Map<Rule, String> map = (Map<Rule, String>) errors.get(field);
		if (map == null) {
			map = new HashMap<Rule, String>();
			errors.put(field, map);
		}
		map.put(rule, error_id);
	}    
	
    /**
     * Implement this abstract method to add rules to the fields you want to validate.
     */
	public abstract void initValidator();
	
    /**
     * By default, the filter returns the <i>BaseAction.ERROR</i> when a validation failure happens.
     * You can change that by calling this method.
     *
     * @param resultForError The result to return in case of a validation failure.
     */
	public void setResultForError(String resultForError) {
		this.resultForError = resultForError;
	}
	
    /**
     * Sets the MessageContext from where you want to get the error messsages.
     * You should only call this method if you want to change the default message context
     * for this filter which is <i>ClassMessageContext(this.getClass(), "/validation")</i>.
     *
     * @param msgContext The MessageContext to use instead of the default one.
     */
    public void setMessageContext(MessageContext msgContext) {
		this.msgContext = msgContext;
	}
    
    protected Action getAction() {
    	
    	return tAction.get();
    }
	
    /**
     * Sets the directory where to look for error messages.
     * You should only call this method if you want to change the default directory,
     * which is <i>/validation</i>.
     * Note that calling this method will force the change of the message context
     * of this filter to ClassMessageContext with the given directory.
     *
     * @param dir The directory where to look for error messages.
     */
	public void setDir(String dir) {
		this.msgContext = new ClassMessageContext(this.getClass(), dir);
	}
	
	public String filter(InvocationChain chain) throws Exception {
		
		Action action = chain.getAction();
		
		tAction.set(action);
		
		Validator validator = getValidator();
		
		boolean isOk = validator.validate(action, msgContext);
		if (!isOk) {
			return resultForError;
		}
		return chain.invoke();
	}
    
    public void destroy() { }
}
		