/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.i18n;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.mentawai.action.BaseLoginAction;

/**
 * @author Sergio Oliveira
 */
public class LocaleManager {
	
	private static String DEFAULT_DIR = "i18n";
	private static String dir = DEFAULT_DIR;	
	private static List<Locale> locales = new ArrayList<Locale>(20);
    private static final String SEP = File.separator;
    private static String master = DEFAULT_DIR + SEP + "master"; // i18n/master
    private static boolean useMasterForEverything = false;
	public static Locale DEFAULT_LOCALE = new Locale("en", "US");
	private static String defaultDateMask = null;
	private static Map<Locale, String> dateMasks = new HashMap<Locale, String>();
	private static boolean masterChanged = false;
	

    
    /**
     * @since 1.1.2
     */
    public static void setUseMasterForEverything(boolean useMaster) {
        useMasterForEverything = useMaster;
    }
    
    private static String cutSlashes(String name) {
    	
        if (name.startsWith("/") && name.length() > 1) {
        	
            name = name.substring(1, name.length());
            
        }
        
        if (name.endsWith("/") && name.length() > 1) {
        	
        	name = name.substring(0, name.length() - 1);
        }
        
        return name;
    }
    
    /**
     * Set the directory from where to look for i18n files. 
     * 
     * Default directory is "i18n" on the root dir. ("/i18n")
     * 
     * To change for WEB-INF/classes/i18n you can call:
     * 
     * LocaleManager.setDir("WEB-INF/classes/i18n");
     * 
     * Notice that this method will also change the location of the master file,
     * if and only if you never called the setMaster method!
     * 
     * @param directory The new directory for the i18n files.
     * @since 1.9
     */
    public static void setDir(String directory) {
    	
    	String newDir = cutSlashes(directory);
    	
    	String oldDir = dir;
    	
    	if (!masterChanged) {
    		
    		// move the master to the new directory as well...
    		
    		master = master.replaceFirst(oldDir, newDir);
    		
    	}
    	
    	dir = newDir;
    }
    
    /**
     * Return the current dir for i18n files.
     * 
     * @return The current dir
     * @since 1.9
     */
    public static String getDir() {
    	
    	return dir;
    }
    
    /**
     * @since 1.1.2
     */
    public static boolean isUseMasterForEverything() {
        return useMasterForEverything;
    }
	
	public static void add(Locale loc) {
        
        Iterator<Locale> iter = locales.iterator();
        
        while(iter.hasNext()) {
            Locale l = iter.next();
            if (l.equals(loc)) return;
        }
        
		locales.add(loc);
	}
	
	public static void setMaster(String s) {
		
		masterChanged = true;
		
		master = s;
        if (master.startsWith("/") && master.length() > 1) {
            master = master.substring(1);
        }
	}
	
	public static String getMaster() {
		return master;
	}
	
	public static void add(String loc) {
		Locale l = getLocaleFromString(loc);
		add(l);
	}
    
    public static Locale getDefaultLocale() {
        if (locales.size() == 0) return DEFAULT_LOCALE;
		return locales.get(0);
	}
	
	public static Locale [] getLocales() {
		Locale [] locs = new Locale[locales.size()];
		locales.toArray(locs);
		return locs;
	}
	
	public static int [] getLocaleIds() {
		int [] ids = new int[locales.size()];
		for(int i=0;i<locales.size();i++) {
			ids[i] = i;
		}
		return ids;
	}

    public static Locale getLocaleFromString(String text) {
        StringTokenizer st = new StringTokenizer(text, "_");
        if (st.countTokens() == 1) {
            return new Locale(st.nextToken());
        } else if (st.countTokens() == 2) {
            return new Locale(st.nextToken(), st.nextToken());
        } else if (st.countTokens() == 3) {
            return new Locale(st.nextToken(), st.nextToken(), st.nextToken());
        }
        return null;
    }
	
	public static boolean isSupportedLocale(Locale loc) {
		Iterator<Locale> iter = locales.iterator();
		while(iter.hasNext()) {
			Locale l = iter.next();
			if (l.equals(loc)) return true;
		}
		return false;
	}
    
    public static boolean isSupportedLocale(String text) {
        Locale loc = getLocaleFromString(text);
		if (loc == null) return false;
		return isSupportedLocale(loc);
    }
    
    public static Locale getLocale(int id) {
        if (id >= locales.size()) return null;
		return locales.get(id);
    }
    
    public static int getId(Locale loc) {
		for(int i=0;i<locales.size();i++) {
			if (locales.get(i).equals(loc)) return i;
		}
		return -1;
    }
    
    public static Locale getSupportedLocale(Locale loc) {
        if (isSupportedLocale(loc)) return loc;
		
		Locale onlyLang = new Locale(loc.getLanguage());
		
		Iterator<Locale> iter = locales.iterator();
		while(iter.hasNext()) {
			Locale l = iter.next();
			if (l.equals(onlyLang)) return l;
        }		
		
		iter = locales.iterator();
		while(iter.hasNext()) {
			Locale l = iter.next();
			if (l.getLanguage().equals(loc.getLanguage())) return l;
        }
        return getDefaultLocale();
    }
    
	public static Locale getLocale(HttpServletRequest req) {
        return getLocale(req, true);
	}
    
	public static Locale getLocale(HttpServletRequest req, boolean onlySupported) {
		Locale loc = null;
		HttpSession session = req.getSession(true);
        Object obj = session.getAttribute(BaseLoginAction.LOCALE_KEY);
		if (obj != null && obj instanceof Locale) {
			loc = (Locale) obj;
		} else {
			loc = req.getLocale();
		}
        if (onlySupported) {
    		return getSupportedLocale(loc);
        } else {
            return loc;
        }
	}  
	
	public static void setDefaultDateMask(String defaultDateMask) {
		
		LocaleManager.defaultDateMask = defaultDateMask;
	}
	
	public static String getDefaultDateMask() {
		
		return defaultDateMask;
	}
	
	public static void setDateMask(Locale loc, String dateMask) {
		
		dateMasks.put(loc, dateMask);
	}
	
	public static String getDateMask(Locale loc) {
		
		String s = dateMasks.get(loc);
		
		if (s == null) return defaultDateMask;
		
		return s;
	}
	
}