/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.list;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * The ListData that will load its ListItems from a database.
 * 
 * All you have to do is provide:
 * 
 * - the name of the key column
 * - the name of the value column
 * - the name of the locale column
 * - the name of the table
 * - the column to sort
 * 
 * Or if you database table is not localized:
 * 
 * - the name of the key column
 * - the name of the value column
 * - the name of the table
 * - the single locale to use for that list 
 * 
 * @author Sergio Oliveira Jr.
 * @since 1.9
 *
 */
public class DBListData implements ListData {
	
	private final LocalizedListData data;
	
	private final SimpleListData noI18nData;
	
	private Map<String, Locale> locales = new HashMap<String, Locale>();
	
	private final String keyColumn, valueColumn, localeColumn, tableName, orderByColumn;
	
	/**
	 * If your database table is localized, in other words, if it has a locale column to define the locale, use this constructor.
	 * 
	 * @param name
	 * @param keyColumn
	 * @param valueColumn
	 * @param localeColumn
	 * @param tableName
	 * @param orderByColumn
	 */
	public DBListData(String name, String keyColumn, String valueColumn, String localeColumn, String tableName, String orderByColumn) {
		
		this.data = new LocalizedListData(name);
		
		this.noI18nData = null;
		
		this.keyColumn = keyColumn;
		
		this.valueColumn = valueColumn;
		
		this.localeColumn = localeColumn;
		
		this.tableName = tableName;
		
		this.orderByColumn = orderByColumn;
	}
	
	/**
	 * If your database table is not localized, in other words, if it does not care about locales and have only one list in one locale, use this constructor.
	 * 
	 * @param name
	 * @param keyColumn
	 * @param valueColumn
	 * @param tableName
	 * @param orderByColumn
	 */
	public DBListData(String name, String keyColumn, String valueColumn, String tableName, String orderByColumn) {
		
		this.data = null;
		
		this.noI18nData = new SimpleListData(name);
		
		this.keyColumn = keyColumn;
		
		this.valueColumn = valueColumn;
		
		this.localeColumn = null;
		
		this.tableName = tableName;
		
		this.orderByColumn = orderByColumn;
	}
	
	public int size() {
		
		if (data != null) {
		
			return data.size();
			
		} else {
			
			return noI18nData.size();
		}
		
	}
	
	public String getValue(String key, Locale loc) {
		
		if (data != null) {
		
			return data.getValue(key, loc);
			
		} else {
			
			return noI18nData.getValue(key, loc);
		}
	}
	
	public String getValue(int key, Locale loc) {
		
		if (data != null) {
		
			return data.getValue(key, loc);
			
		} else {
			
			return noI18nData.getValue(key, loc);
		}
		
	}
	
	public List<ListItem> getValues(Locale loc) {
		
		if (data != null) {
			
			return data.getValues(loc);
			
		} else {
		
			return noI18nData.getValues(loc);
			
		}
	}
	
	public String getName() {
		
		if (data != null) {
		
			return data.getName();
			
		} else {
			
			return noI18nData.getName();
			
		}
	}
	
	/**
	 * Override this method to map whatever you have in your database to a Java locale.
	 * 
	 * Ex: 
	 * 
	 * pt_BR can return new Locale("pt_BR") (default implementation)
	 * 
	 * "1" = new Locale("pt_BR");
	 * "2" = new Locale("en_US");
	 * etc...
	 * 
	 * @param loc
	 * @return The locale corresping to the String loc in your database
	 */
	protected Locale getLocaleFromString(String loc) {
		
		StringTokenizer st = new StringTokenizer(loc, "_");
		
		int x = st.countTokens();
		
		if (x == 1) {
			
			return new Locale(st.nextToken());
			
		} else if (x == 2) {
			
			return new Locale(st.nextToken(), st.nextToken());
			
		} else if (x == 3) {
			
			return new Locale(st.nextToken(), st.nextToken(), st.nextToken());
			
		} else {
			
			throw new IllegalArgumentException("Bad locale: " + loc);
		}
	}
	
	/**
	 * Call this method passing a db connection to load the lists from database!
	 * 
	 * The SQL statement that will be constructed:
	 * 
	 * "select " + keyColumn + "," + valueColumn + "," + localeColumn + " from " + tableName + " order by " + orderByColumn;
	 * 
	 * or if localeColumn is null and a single locale (unique) is provided
	 * 
	 * "select " + keyColumn + "," + valueColumn + " from " + tableName + " order by " + orderByColumn;
	 * 
	 * @param conn
	 * @throws SQLException
	 */
	public void load(Connection conn) throws SQLException {
		
		String sql;
		
		if (data != null) {
			
			sql = "select " + keyColumn + "," + valueColumn + "," + localeColumn + " from " + tableName + " order by " + orderByColumn;
			
		} else {
			
			sql = "select " + keyColumn + "," + valueColumn + " from " + tableName + " order by " + orderByColumn;
		}
		
		PreparedStatement stmt = null;
		ResultSet rset = null;
		
		try {
			
			stmt = conn.prepareStatement(sql);
			
			rset = stmt.executeQuery();
			
			while(rset.next()) {
				
				String key = rset.getString(1);
				
				String value = rset.getString(2);
				
				Locale locale;
				
				if (data != null) {
					
					String loc = rset.getString(3);
					
					locale = locales.get(loc);
					
					if (locale == null) {
						
						locale = getLocaleFromString(loc);
						
						if (locale == null) throw new IllegalArgumentException("method getLocaleFromString returned null for: " + loc);
						
						locales.put(loc, locale);
					}
					
					data.add(key, value, locale);

				} else {
					
					noI18nData.add(key, value);
				}
			}
			
		} finally {
			
			try { if (rset != null) rset.close(); } catch(Exception e) { }
			try { if (stmt != null) stmt.close(); } catch(Exception e) { }
		}
	}
	
	/**
	 * The only difference of this method to the load(conn) method is that it
	 * will clear everything before loading...
	 * 
	 * @param conn
	 * @throws SQLException
	 */
	public void refresh(Connection conn) throws SQLException {
		
		data.clear();
		
		locales.clear();
		
		load(conn);
	}
}

